@@ -13,6 +13,7 @@ from octopus_python_client.migration import Migration
 from octopus_python_client.processes import clone_process_step, delete_process_step
 from octopus_python_client.projects import clone_project, delete_project, get_project, project_update_variable_sets
 from octopus_python_client.release_deployment import ReleaseDeployment
+from octopus_python_client.utilities.helper import log_raise_value_error
 
 logging.basicConfig(filename=os.path.join(os.getcwd(), "octopus_python_client.log"),
                     filemode='a',
@@ -73,6 +74,7 @@ def _parse_args():
                         help="password for octopus; either api_key or user_name and password are required")
     parser.add_argument("-a", "--action", help=str(Actions.__dict__.values()), required=True)
     parser.add_argument("-ow", "--overwrite", help="if present, overwrite = True", action='store_true')
+    parser.add_argument("-ns", "--no_stdout", help="if present, no_stdout = True, means no stdout", action='store_true')
     parser.add_argument("-ts", "--item_types",
                         help='if not item_types and not octopus_space_id, get all item types '
                              'regardless whether they are above Spaces; if (not item_types) and octopus_space_id, '
@@ -95,13 +97,11 @@ def _parse_args():
     parser.add_argument("-bs", "--base_step_name", help="base step name as in octopus process for cloning a step")
     parser.add_argument("-ps", "--prev_step_name",
                         help="previous step name in octopus process for the step insertion location")
-    parser.add_argument("-sf", "--suffix", help="variable sets name suffix")
-    parser.add_argument("-rs", "--remove_suffix",
-                        help="if present, remove suffix from variable sets name, otherwise add suffix",
-                        action='store_true')
-    parser.add_argument("-ti", "--task_id", help="task id, like ServerTasks-1234")
     parser.add_argument("-tl", "--time_limit_second", help="time limit in second")
     parser.add_argument("-rv", "--release_version", help="release version for creating a new release")
+    parser.add_argument("-as", "--add_suffix",
+                        help="if present, add suffix to variable sets name")
+    parser.add_argument("-rs", "--remove_suffix", help="if present, remove suffix from variable sets name")                        
     parser.add_argument("-pj", "--project_name", help="project name")
     parser.add_argument("-cn", "--channel_name", help="channel name")
     parser.add_argument("-nt", "--notes", help="notes")
@@ -137,30 +137,32 @@ def run():
         config.user_name = args.user_name
     if args.password:
         config.password = args.password
-
     if not config.api_key and not (config.user_name and config.password):
         raise ValueError(f"either api_key or user_name and password are required")
 
-    print('config.octopus_endpoint: ' + config.octopus_endpoint)
-    print('config.octopus_name: ' + config.octopus_name)
-    print('config.current_path: ' + config.current_path)
+    if args.overwrite:
+        config.overwrite = args.overwrite
+    if args.no_stdout:
+        config.no_stdout = args.no_stdout
+
+    logger.info(config.__dict__)
 
     # verify space id/name
     dst_space_id = None
     if args.dst_space_id_name:
         dst_space_id = verify_space(space_id_name=args.dst_space_id_name)
         if dst_space_id:
-            print(f"destination space_id is: {dst_space_id}")
+            logger.info(f"destination space_id is: {dst_space_id}")
         else:
-            raise ValueError(f"the destination space id/name {args.dst_space_id_name} you specified does not exist "
-                             f"or you may not have permission to access it")
+            raise ValueError(f"the destination space id/name {args.dst_space_id_name} you specified does not exist or "
+                             f"you may not have permission to access it")
 
     space_id = None
     fake_space = False
     if args.space_id_name:
         space_id = verify_space(space_id_name=args.space_id_name)
         if space_id:
-            print(f"octopus space_id is: {space_id}")
+            logger.info(f"octopus space_id is: {space_id}")
         elif args.action not in clone_space_actions:
             raise ValueError(f"the space id/name {args.space_id_name} you specified does not exist "
                              f"or you may not have permission to access it")
@@ -173,9 +175,6 @@ def run():
             and input(f"Are you sure you want to run a command against null space [Y/n]? ") != 'Y':
         return
 
-    if args.overwrite:
-        config.overwrite = args.overwrite
-
     if args.action == Actions.action_get_spaces:
         get_spaces_save(item_types_comma_delimited=args.item_types, space_id_or_name_comma_delimited=args.spaces)
     elif args.action == Actions.action_get_types:
@@ -227,8 +226,8 @@ def run():
     elif args.action == Actions.action_get_project:
         get_project(project_literal_name=args.project_name, space_id=space_id)
     elif args.action == Actions.action_project_update_variable_sets:
-        project_update_variable_sets(project_literal_name=args.project_name, suffix=args.suffix, space_id=space_id,
-                                     remove_suffix=args.remove_suffix)
+        project_update_variable_sets(project_literal_name=args.project_name, space_id=space_id,
+                                     remove_suffix=args.remove_suffix, add_suffix=args.add_suffix)
     elif args.action == Actions.action_clone_space:
         Migration().clone_space(src_space_id=space_id, dst_space_id=dst_space_id,
                                 item_types_comma_delimited=args.item_types, fake_space=fake_space)
@@ -236,9 +235,9 @@ def run():
         Migration().clone_space_item(src_space_id=space_id, dst_space_id=dst_space_id, item_type=args.item_type,
                                      item_name=args.item_name, item_id=args.item_id, fake_space=fake_space)
     elif args.action == Actions.action_task_status:
-        get_task_status(task_id=args.task_id, space_id=space_id)
+        get_task_status(task_id=args.item_id, space_id=space_id)
     elif args.action == Actions.action_wait_task:
-        wait_task(task_id=args.task_id, space_id=space_id, time_limit_second=args.time_limit_second)
+        wait_task(task_id=args.item_id, space_id=space_id, time_limit_second=args.time_limit_second)
     elif args.action == Actions.action_create_release:
         ReleaseDeployment.create_release_direct(
             release_version=args.release_version, project_name=args.project_name, channel_name=args.channel_name,
@@ -256,7 +255,7 @@ def run():
             tenant_name=args.tenant_name, comments=args.comments)
 
     else:
-        raise ValueError("We only support operations: " + str(Actions.__dict__.values()))
+        log_raise_value_error(local_logger=logger, err="We only support actions: " + str(Actions.__dict__.values()))
 
 
 def main():
